﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;
using System.IO.Ports;
using System.IO;

namespace RemDisplay
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        const int timeout = 100;   // timeout 10 sec, (100msec base)

        DispatcherTimer timer1;
        SerialPort port;
        string rxbuf = "";
        int toPwr = 0;  // timeouts
        int toSwr = 0;
        double[] pwrBuf = new double[10];
        bool warning = false;
        string inifile = @"RemDisplay.ini";
        string power = "";
        string swr = "";
        double powerbar = 0;
        double swrbar = 0;

        public Window1()
        {
            InitializeComponent();

            try
            {
                TextReader tr = new StreamReader(inifile);
                string portname = tr.ReadLine();
                tr.Close();
                foreach (object x in cbComPort.Items)
                {
                    ComboBoxItem cbi = (ComboBoxItem)x;
                    if (cbi.Content.ToString() == portname)
                    {
                        cbComPort.SelectedItem = cbi;
                        break;
                    }
                }
            }
            catch { }

            timer1 = new DispatcherTimer();
            timer1.Interval = new TimeSpan(1000000);    // 100 msec
            timer1.Tick += new EventHandler(timer1_Tick);
            timer1.Start();

        }

        void port_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            int chr;
            try
            {
                while ((chr = port.ReadByte()) >= 0)
                {
                    if ((chr == 0x0a) || (chr == 0x0d))
                    {
                        ProcessData(rxbuf);
                        rxbuf = "";
                    }
                    else
                        rxbuf += (char)chr;
                }
            }
            catch { }
        }

        void ProcessData(string data)
        {
            data = data.Trim();
            if (data.Length < 2)
                return;
            if (data.StartsWith("Pw"))
            {
                power = data.Substring(2);
                toPwr = 0;
                string value = "";
                int i = 0;
                while ((i < 10) && ((power[i] >= '0') && (power[i] <= '9') || (power[i] == '.')))
                {
                    value += power[i++];
                }

                try
                {
                    double pwr = Double.Parse(value);
                    double maxPower = 0.0;
                    for (i = 0; i < 9; i++)
                    {
                        pwrBuf[i] = pwrBuf[i + 1];  // shift buffer
                        if (pwrBuf[i] > maxPower)
                            maxPower = pwrBuf[i];
                    }
                    pwrBuf[9] = pwr;  // enter Fifo buffer
                    if (pwr > maxPower)
                        maxPower = pwr;

                    powerbar =(maxPower > 0)? pwr/maxPower*100: 0.0;  // bar indicator
                }
                catch { }
            }
            else if (data.StartsWith("SWR"))
            {
                swr = data.Substring(3);
                toSwr = 0;
                try
                {
                    double swrd = Double.Parse(swr);
                    swrbar = swrd > 1 ? 100 - 200 / (swrd + 1) : 0.0;   // SWR bar
                }
                catch { }
            }
        }

        void timer1_Tick(object sender, EventArgs e)
        {
            if ((port == null) || !port.IsOpen)
            {
                string sel = cbComPort.SelectionBoxItem.ToString();
                port = new SerialPort(sel, 9600);
                port.DataReceived += new SerialDataReceivedEventHandler(port_DataReceived);
                rxbuf = "";
                for (int i = 0; i < 10; i++)
                    pwrBuf[i] = 0.0;
                try
                {
                    port.Open();
                }
                catch (Exception ex)
                {
                    if (!warning)
                    {
                        MessageBox.Show(ex.Message, "Error");
                        warning = true;
                    }
                }
            }

            if (toPwr++ >= timeout)
            {
                toPwr = timeout;
                lbPwr.Content = "???";
                barPwr.Value = 0.0;
            }
            else
            {
                lbPwr.Content = power;
                barPwr.Value = powerbar;
            }

            if (toSwr++ >= timeout)
            {
                toSwr = timeout;
                lbSwr.Content = "???";
                barSwr.Value = 0.0;
            }
            else
            {
                lbSwr.Content = swr;
                barSwr.Value = swrbar;
            }
        }

        private void cbComPort_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            warning = false;    // warning box will be shown upon next event
            if (port != null)   // if connected, disconnect old port
            {
                if (port.IsOpen)
                    port.Close();
                port = null;
            }
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (port != null)   // if connected, disconnect
            {
                if (port.IsOpen)
                    port.Close();
                port = null;
            }

            try
            {
                TextWriter wr = new StreamWriter(inifile);
                wr.WriteLine(cbComPort.SelectionBoxItem.ToString());
                wr.Close();
            }
            catch { }
        }
    }
}
